ol, ul {list-style: none outside none;}
blockquote, q {quotes: none;}
blockquote:before, blockquote:after, q:before, q:after {content: none;}
:focus {outline: 0 none;}
img{border:0}

/*YAHOO RESET FONT*/
html{color:#000;}body,div,dl,dt,dd,ul,ol,li,h1,h2,h3,h4,h5,h6,pre,code,form,fieldset,legend,input,button,textarea,select,p,blockquote,th,td{margin:0;padding:0}table{border-collapse:collapse;border-spacing:0}fieldset,img{border:0}address,button,caption,cite,code,dfn,em,input,optgroup,option,select,strong,textarea,th,var{font:inherit}del,ins{text-decoration:none}li{list-style:none}caption,th{text-align:left}h1,h2,h3,h4,h5,h6{font-size:100%;font-weight:normal}q:before,q:after{content:''}abbr,acronym{border:0;font-variant:normal}sup{vertical-align:baseline}sub{vertical-align:baseline}legend{color:#000}body{font:13px/1.231 arial,helvetica,clean,sans-serif;*font-size:small;*font:x-small}select,input,textarea,button{font:99% arial,helvetica,clean,sans-serif}table{font-size:inherit;font:100%}pre,code,kbd,samp,tt{font-family:monospace;*font-size:108%;line-height:100%}
body{
  background: #6A93A0;/*#8DABB4;/* #A2BDC5;*/
  height:100%;
}

a{
    text-decoration: none;
    color: #19485C;
}

a:hover {
    color: #4DA0C6;
}

table {
    margin: 0;padding:0;
    border-right: none;
    border-bottom: none;
    font-size: 14px;
    background: #fff;
    border-spacing:0;
}
td{
    padding: 4px;
    border: none;
    border-bottom: 1px solid #A8A8A8;
    border-right: 1px solid #A8A8A8;
}
td.propertie{padding-left:10px; color:blue;width: 178px;}
.slapvalues{display:block;width: 522px;word-wrap: break-word}
td.first{border-left: 1px solid #A8A8A8;}
th{
    background: #2281C1;
    padding: 6px;
    color: #fff;
    font-weight: normal;
    font-size: 18px;
    border-right: 1px solid #2281C1;
}
table.small th{padding: 4px;font-size: 16px;}
textarea {
  width:932px;
  font-family: 'Arial,Helvetica Neue',Tahoma,Helvetica,sans-serif;
}

body {
	color: #000000;
  overflow: -moz-scrollbars-vertical;
  overflow-y: scroll;
}

#page
{
	margin: 10px auto;
	width: 1032px;
}

#logo{
	margin-left:60px;
}

#header{
    height: 78px;
    padding-top: 10px;
    width: 1032px;
    background: url(../images/head.png) no-repeat;
}

.block_header{
    text-align: left;
    padding-left: 20px;
    height: 30px;
    position:relative;
}

#header .run{
    margin-right: 20px;
    float: right;
}
#running p{
  float:left;
  padding: 5px;
  padding-right: 10px;
  border-left: 4px solid #C753C4;
  font-size: 1.08em;
  color: #2D73B1;
}
#running p.software{
  border-left: 4px solid #1E73BD;
}
#running img{
  float:left
}

#header .info{
    font-size: 16px;
    font-weight: normal;
    padding-top: 3px;
    margin-left:20px;
    float: left;
    width: 695px;
    height: 22px;
    text-align: center;
    color: #4c6172;
    text-shadow: 0 1px 1px #DBDBDB;
    overflow: hidden;
}
#header .line{
    width:1px;
    border-left: 1px solid #678DAD;
    background: #CAD4DC;
    height: 24px;
    float: left;
    margin-top: 2px;
    margin-right: 5px;
     margin-left:5px
}

#wmenu{
  margin: 1px 7px 1px 10px;
}

#wmenu ul{
    width: 100%;
}

.space{
    margin-left: 28px;
}

#main
{
    background: url(../images/main_bg_all.png) repeat-y;
    padding-left: 17px;
    padding-right: 17px;
    min-height: 400px;
}

.main_content{
    position:relative;
    /*width: 99.6%;*/
    height: 430px;
    margin-top:2px;
    border:1px solid #519ADA;
    overflow:auto;
}

.main_head{
    height: 10px;
    background: url(../images/main_top.png) no-repeat;
    width: 998px;
}

.content{
    background: url(../images/content.png) repeat-y;
    width: 978px;
    padding: 0 10px;
    min-height: 400px;
}

.content h2{
    color: #526063;
    font-weight: normal;
    font-size: 1.25em;
    line-height: 25px;
}
.content h2 a{
    text-decoration:none;
    color: #4DA0C6;
}
.content h2 a:hover{
    color: #19485C;
}

.form-elt{
  min-width: 100px;
  display: block;
  float:left;
  margin-top: 10px;
  margin-right: 10px;
  line-height: 22px;
}

.main_foot{
    height: 10px;
    background: url(../images/main_bottom.png) no-repeat;
    width: 998px;
}

#footer{
    height: 33px;
    width: 952px;
    background: url(../images/footer_bg.png) no-repeat;
    padding-left: 40px;
    padding-right: 40px;
    padding-top: 7px;
    text-align: center;
    color: #4c6172;
    text-shadow: 0px 1px 1px #DBDBDB;
}
.dsblebutton{color:#9C9C9C; padding: 5px 10px;display: inline;background: #eee;
        font-weight: bold; border:1px solid #000; margin-top:10px;}
.button {
    padding: 5px 10px;
    display: inline;
    background: #eee;
    color: #000;
    cursor: pointer;
    font-weight: bold;
    border:1px solid #000;
    margin-top:10px;
}
.button:hover {
	background-position: 0 center;
	background: #666666;
	color: #fff;
	box-shadow: 1px 1px 1px #888888;
}
.button:active {
  background: #eee;
  color: #000;
}

.focusField{
    border:solid 1px #73A6FF;
    background:#EFF5FF;
    color:#000;
    padding:5px;
    outline:none;
    font: 13px 'Helvetica Neue', Helvetica, Arial, sans-serif;
}
.idleField{
    background:#fff;
    color: #6F6F6F;
    border: solid 1px #678dad;
    padding:5px;
    outline:none;
    font: 13px 'Helvetica Neue', Helvetica, Arial, sans-serif;
}
.fancytree-edit-input{background:#fff; border: solid 1px #678dad; outline:none; padding:2px;}

input[type="radio"], input[type="checkbox"]{
    margin: 4px;
}

.flash{
    margin-left:20px;
    color:#4A131F;
    font-style:italic;
    font-size:14px;
    position:relative;
}

.flashes{
	margin-bottom:10px;
}

#commitmsg {
	width:95%;
}

.message {
	color:#FF5500;
	line-height:21px;
}

.clear{
    clear: both;
    height:1px;
}

.home_content{

    padding: 15px;
}

#home_box{
    background: none;
    border: 2px solid #87B0D4;
    padding: 0;
    color: #4c6172;
    margin: 20px auto;
    width: 641px;
}

#home_box a{
    color: #475F73;
    cursor: pointer;
    text-decoration: none;
}

#home_box a:hover{
    text-decoration: underline;
}

#home_box h2{
    font-weight: normal;
    font-size: 23px;
    color: #475F73;
}

#home_box p{
    font-size: 13px;
    color: #475F73;
    float: left;
}

#home_box img{
    border: none;
    float: left;
    margin-left: 10px;
}

.inner_box{
    background: none;
    margin: 0;
    display: block;
    padding: 30px 0 30px 0;
}

.inner_box p{
    font-size: 13px;
    float: left;
    width: 450px;
    cursor:default;
    text-align:justify;
}


.lmenu{
    display: block;
    height: 80px;
    margin: 10px 0 10px 45px;
    padding: 10px;
    padding-bottom:15px;
    width: 530px;
    border: 1px solid #678dad;
}

.smaller{margin-left:10px; width: 350px; float:left}
.smaller p{width: 280px;}
.umenu{display: block;margin: 10px 0 10px 45px; width: 147px; float:left;border: 1px solid #678dad;
padding: 10px;height: 80px;padding-bottom:15px;}
.umenu p{font-size: 13px;float: left;width: 77px;cursor:default;}
.lmenu:hover, .smenu:hover, .sright_menu:hover, .smaller:hover, .umenu:hover{
    background: #96b9d7;
}
.smenu{
    display: block;
    height: 80px;
    margin: 0 10px 0 45px;
    padding: 10px;
    padding-bottom:15px;
    width: 250px;
    border: 1px solid #678dad;
    float: left;
}

.sright_menu{
    display: block;
    height: 80px;
    padding: 10px;
    padding-bottom:15px;
    border: #678dad;
    width: 247px;
    border: 1px solid #678dad;
    float: left;
}

.sright_menu p, .smenu p{
    font-size: 13px;
    float: left;
    width: 170px;
    cursor:default;
}

.file_tree {
    /*width: 923px;*/
    height: 250px;
    border: solid 1px #678dad;
    background: #fff;
    padding: 5px;
}

.file_tree_tabs {
    width: 702px;
    height: 250px;
    border: solid 1px #678dad;
    background: #fff;
    padding: 5px;
}

.file_tree_short{
    width: 252px;
    height: 429px;
    border: solid 1px #678dad;
    background: #fff;
    padding: 0px;
}
.box_software{
    width: 240px;
    background: #fff;
}
.software_details{float: left;}
#details_box{
    display:block;
    margin-top:2px;
}
#code{
    float: right;
    width: 720px;
}
#details_head{margin-bottom: 10px;}

#contentInfo{
    border: solid 1px #678dad;
    background: #fff;
    overflow: auto;
    padding: 3px;
}

#contentInfo p{
    font-size: 14px;
    margin: 5px;
}

.gradient{background-color:#f4f4f4;background:-moz-linear-gradient(#f4f4f4,#ececec);background:-webkit-linear-gradient(#f4f4f4,#ececec);-ms-filter:"progid:DXImageTransform.Microsoft.gradient(startColorstr='#f4f4f4',endColorstr='#ececec')";background:linear-gradient(#f4f4f4,#ececec);}

#slapstate {
   display: none;
}

.log_content{
    border: solid 1px #519ADA;
    padding: 2px;
    width: 761px;
    float: left;
}

.log_information{
  border: solid 1px #519ADA;
  padding: 2px;
  width: 200px;
  float: left;
  margin-left: 5px;
}

.log {
    border: none;
    background: #f6f7f8;
    color: #000;
    padding: 2px;
    height: 450px;
    resize: both;
    /*font-size: 0.98em;*/
    overflow-y: auto;
    white-space: pre-wrap;
}
.log_btn{border: 1px solid #678dad; border-top:none; height: 22px; width: 654px; margin: 0 40px 10px 40px;
    padding:4px 10px 4px 10px; font-size: 16px; color: #4c6172;}
.log p{white-space: pre-wrap; width:100%}
.log p.info{white-space: pre-wrap; width:98%; font-size: 15px; margin: 5px; color:red;}

.log_info_box{
  height: 454px;
  width: 200px;
}
.log_info_box h2{
  font-weight: bold;
  padding: 2px;
  padding-left: 20px;
  text-align: left;
  color: #fff;
  display: block;
  font-size: 16px;
  margin-top: 20px;
}
.log_info_box h2:first-child{
  margin-top: 0;
}

.log_info_box h2.software, h2.boxtitle{
  background: #1E73BD;
}
.log_info_box h2.instance{
  background: #C753C4;
}
.log_info_box ul li{padding: 5px; font-size: 14px;color: #777B7C; font-weight: bold; margin: 3px 0; cursor: pointer;border: 1px solid #fff}
.log_info_box ul li:hover{background: #EBEBEB}
.log_info_box ul li.checked{border: 1px solid #DADADA}
.add_btn{background: url(../images/plus.png) center left no-repeat; display: block; margin: 2px; opacity: 0.6; min-width:22px; height: 22px;
    text-indent: 30px; font-weight: bold; padding-top: 7px;}
.add_btn:hover{opacity: 1;}

#software_run_state, #instance_run_state{
  color: #000;
  font-weight: bold;
  margin-bottom: 10px;
}
#software_run_state p, #instance_run_state p{
  padding-top: 2px;
  font-size: 16px;

}
.last_build{
  font-size: 0.9em;
}
.log_content_box{
  padding: 10px 5px;
  font-size: 14px;
  min-height: 130px;
}
.log_content_box p{
  line-height: 20px;
}
.log_content_box a{
  color: #2084CA;
  font-weight: bold;
}
.log_content_box a:hover{
  color: #286d9e;
}
.log_info_box span.legend{
  height: 22px;
  width: 24px;
  margin-right: 10px;
  display: block;
  float: left;
}
.log_info_box div.state_running span, .log_info_box div.state_notupdated span{
  background: #FFC800;
}
.log_info_box div.state_terminated span{
  background: #00C800;
}
.log_info_box div.state_waiting span, .log_info_box div.state_stopped span, .log_info_box div.state_failed span{
  background: #FF3131;
}

.separator{
  border-bottom: 3px solid #D6D6D6;
  width: 100%
}
.space{height: 5px;width: 100%;}

.waitting{
    margin-left: 10px;
    display: none;
}

.file_info{
    margin-top:5px;
    background: #e4e4e4;
    padding: 5px 10px 5px;
    box-shadow: 1px 1px 1px #888888;
    color: #737373;
    font-weight: bold;
    text-shadow: 0px 1px #FFF;
    font-size: 14px;
    cursor:default;
    word-wrap: break-word
}

#check{
    margin-right:5px;
    margin-top:2px;
    display: none;
    float: left;
}

.public_key{
    border: 1px solid #678dad;
    overflow: auto;
    height: 95px;
    padding: 5px;
    background:#fff;
    margin-bottom: 10px;
    color: #3A494F;
    font-size:  12px;
}

.div{
    border-top: 1px solid #678dad;
    height: 1px;
    width: 734px;
    margin: 10px;
    margin-top: 20px;
}
label.header {
    color: #4C6172;
    font-size: 18px;
    font-weight: normal;
}
select {
  -webkit-border-radius: 2px;
  -moz-border-radius: 2px;
  box-shadow: 0px 1px 3px rgba(0, 0, 0, 0.1);
  -webkit-padding-end: 20px;
  -webkit-padding-start: 2px;
  -webkit-user-select: none;
  background: #fff;
  border: 1px solid #678dad;
  color: #000;
  font-size: inherit;
  margin: 0;
  overflow: hidden;
  padding: 4px;
  text-overflow: ellipsis;
  white-space: nowrap;
  min-width: 80px;}

a.lshare{
    background: #fff;
    padding: 0.27em .47em;
    height: 18px;
    font-size: 1.14em;
    border: solid 1px #678dad;
    color: #4DA0C6;
    text-align:center;
    font-weight:bold;
    cursor:pointer;
}
a.lshare:hover{
    background:#D9D9D9;
    border:solid 1px #73A6FF;
    text-shadow: 0px 1px #F0F1F2;
    color: #666666;
}
a.lshare:focus{
    border:solid 1px #73A6FF;
}
a.no-right-border{border-right:none}
a.no-right-border:hover{border-right:none}
a.no-right-border:focus{border-right:none}
a.lshare img{
    margin: 5px;
}

#editor, #editorViewer {
    margin: 0;
    position: absolute;
    top: 0;
    bottom: 0;
    left: 0;
    right: 0;
}

#error{
    padding: 0px;
    position: absolute;
    top: 0px;
    left:0;
    right: 30px;
}

#error div{
    background: none repeat scroll 0 0 #FFC8F4;
    border: 1px solid #A70073;
    padding: 5px;
    margin: 5px 0;
    min-width: 300px;
    max-width: 500px;
    float: right;
    border-radius: 2px;
    box-shadow: 0 2px 4px rgba(0, 0, 0, 0.2);
    text-align: left;
    position: relative;
    z-index: 170;
    clear: both;
}
#error div.alert{background: lightYellow; border: 1px solid #E6DB55;}
#error div.confirm, #error div.info{background: none repeat scroll 0 0 #CBECF7; border: 1px solid #2599A3;}

#error table{
    background: none;
    width: 100%;
    font-weight: normal;
}
#error table, #error th, #error tr, #error td{
    border: none;
    padding:0;
}

#error td{
    margin:2px;
}

#error td.logo{
    width: 36px;
    height: 26px;
}
#error td.b_close{
    width: 30px;
    height:22px;
}
#error p{white-space: pre-wrap; word-wrap: break-word;padding: 0 5px; line-height: 18px; color: #67767C;max-width: 430px;}
.noscroll {
    overflow: hidden;
}
.form{padding:10px; padding-left:20px;}
.form label{display:block; float:left; width:150px; padding-top:10px;}
.form input[type=text] ,.form input[type=password], .form select {float:left; width:190px;margin:5px;}
.hiddendiv {display: none;white-space: pre-wrap;min-height: 18px;font-size: 13px;
  padding:3px;word-wrap: break-word;width:430px; max-height:120px;font-family: 'Helvetica Neue',Tahoma,Helvetica,Arial,sans-serif;}
.list{display: block;padding: 5px;background: #E7E7E7; margin-bottom:7px;}
div.sep{display:block; height:7px;}
.slidebox{padding:10px; }

.pClose, .close{background:url(../images/close.png) no-repeat 0px 0px; display:block; width:22px; height:22px; cursor:pointer}
.pClose:hover, .close:hover{background:url(../images/close_hover.png) no-repeat 0px 0px;}

.md5sum {margin:10px; font-size:15px;}

#contentInfo h2, .hight, .title{background: #D5D5D5; width: 100%; height: 26px; padding-top:2px; text-indent: 5px; color: #526063;
    text-shadow: 0px 1px #ECECEC;}
.hight{border-bottom: 3px solid #6DB9DD;}
.show{background:#D5D5D5 url(../images/arrow_up.png) 97% 50% no-repeat; cursor: pointer;}
.hide{background:#D5D5D5 url(../images/arrow_down.png) 97% 50% no-repeat; cursor: pointer;}

.menu-box-left{float:left; width: 162px; font-size:14px; background: #e4e4e4; padding:5px 0 5px 5px; margin-top:10px;
    font-size:14px; border-radius: 4px 0 0 4px;}
.menu-box-right{background: #e4e4e4; padding: 5px; float:right; width: 760px; margin-left: -5px;
    margin-top:10px; box-shadow: 1px 1px 1px #888888;}
.menu-box-left ul{margin:0px; list-style:none;}
.menu-box-left li{padding: 2px; padding-left:10px; padding-right:10px; text-shadow: 0px 1px #fff;border-bottom:1px solid #fff;}
.menu-box-left li label, .menu-box-left li input[type=radio] { cursor:pointer}
.menu-box-left li:hover{background:#F0F2F2;}
.menu-box-left li.checked{background:#fff;}
.menu-box-right h2{text-align:center}
/* Bubble pop-up */
.popup {position: absolute; display: none; z-index: 150;}
.popup table, .popup td, .popup th{margin:0; padding:0; background:none; border:0; border-collapse:collapse;}
.popup td.corner {height: 15px;width: 19px;}
.popup td#topleft { background-image: url(../images/bubble-8.png); }
.popup td.top { background-image: url(../images/bubble-7.png);}
.popup td#topright { background-image: url(../images/bubble-6.png); }
.popup td.left { background-image: url(../images/bubble-4.png); }
.popup td.right { background-image: url(../images/bubble-5.png); }
.popup td#bottomleft { background-image: url(../images/bubble-3.png); }
.popup td.bottom { background-image: url(../images/bubble-2.png);}
.popup td.top img { display: block; }
.popup td#bottomright { background-image: url(../images/bubble-1.png); }
.popup-contents {background: #fff; color: #666666;}
.popup a{display: block; color: #4DA0C6; padding-left: 5px; padding-bottom: 5px;}
.popup a:hover{color: #666666;}
.popup ul{margin:0; margin-bottom:10px; padding:0; list-style:none;}
.popup li{border-bottom: 1px solid #E4E4E4; padding:3px 0; margin: 0 5px;}
.popup ul li a{padding-bottom:0}
ul.menu li {padding-top:5px; padding-bottom:5px;}
ul.menu{margin-bottom:0;}
.inline a, .inline{display: inline;}
.popup-value{display:none;}
textarea.parameter {border: solid 1px #678dad; color: #666666; height:110px;}
.link{color:#fff; font-weight:bold; text-decoration:none}
.link:hover{color: #19485C;}
input[type=radio] {
}

input[type=radio]:checked {
}

input[type=radio]:hover {
    box-shadow: 0px 1px 3px #F0F1F2;
}
/* Login Css region *******/
#login-page{width:429px; height:236px; margin:130px auto 0px; background:url(../images/loginBox.png) no-repeat;
padding:10px; font-size:14px; color:#03406A}
#login-page h2{color:#fff; font-size:26px; font-weight:normal; text-indent:50px;}
.login-content{position:relative;margin:10px; margin-top:30px; margin-bottom:0;}
.login-button{width:140px; margin:0 auto;}
.login-element{float:left; min-width:120px;}
.login-label{padding:5px; font-size:16px;}
.login-input{width:220px;}
.information{display:block; float:left; height:16px; margin-top:10px; margin-left:10px; font-weight:bold}
.account{margin-left:60px;}

/*-------------------------ACE UPDATE------------------------*/
.ace_search{
  width: 350px;
  max-width: 350px;
}

.bt_close {
    text-decoration: none;
    color: #999;
    font-weight: bold;
    font-size: 1em;
    padding: 0 4px;
    -webkit-border-radius: .2em;
    -moz-border-radius: .2em;
    border-radius: .2em;
    cursor: pointer;
}
.bt_close:hover {
    color: #333;
}

.fullScreen{
  background: #fff;
  height: auto!important;
  width: auto!important;
  border: 0;
  margin: 0;
  position: fixed !important;
  top: 0;
  bottom: 0;
  left: 0;
  right: 0;
  z-index: 140;
}
.fullScreen-editor{
  height: auto!important;
  width: auto!important;
  position: fixed !important;
  top: 35px;
  bottom: 0;
  left: 244px;
  right: 0;
}

.fullScreen-tree{
  height: auto!important;
  width: 238px;
  position: fixed !important;
  top: 37px;
  bottom: 0;
  left: 0;
  right: 0;
}

.fullScreen {
  overflow: hidden;
}

/*  *******************  */
#shellinabox{width:100%; min-height:530px;}
