{% extends "layout.html" %}
{% block title %}View log{% endblock %}
{% block head %}
  {{ super() }}
  <script src="{{ url_for('static', filename='js/scripts/viewlog.js') }}" type="application/javascript" charset="utf-8"></script>
  <link href="{{ url_for('static', filename='css/ui.fancytree.css', _external=False) }}" rel="stylesheet" type="text/css" media="screen" />
  <script src="{{ url_for('static', filename='js/jquery/jquery.fancytree.min.js') }}" type="application/javascript" charset="utf-8"></script>
  <link href="{{ url_for('static', filename='css/colorbox.css', _external=False) }}" rel="stylesheet" type="text/css" media="screen" />
  <script src="{{ url_for('static', filename='js/jquery/jquery.colorbox-min.js') }}" type="application/javascript" charset="utf-8"></script>
{% endblock %}
{% block body %}


<h2 class="hight hide" id="logheader">Inspecting slapgrid log - Click for more options</h2>
<div class="log_btn" id="logconfigbox" style="display:none">
<span style="margin-left:15px; font-size: 18px;"> Update parameters </span>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
<input type="radio" name="liveupdate" id="manual" value="manual" />
<label for="manual">Manualy update</label>&nbsp;
<input type="radio" name="liveupdate" id="live" value="live" />
<label for="live">Live update</label>&nbsp;
<input type="radio" name="liveupdate" id="slow" value="slow" />
<label for="slow">From time to time</label>
</div>
<div class='space'></div>
<div class="log_content">
  <div class="log" id="salpgridLog"></div>
  <div class="log" id="manualLog" style="display:none"></div>
</div>
<div class="log_information">
  <div class="log_info_box">
    <div id="slapstate">
      <h2 class="software">Building State</h2>
      <div class="log_content_box">
         <div id="software_run_state" class="state_running">
          <span class="legend"></span>
          <p>Processing</p>
          <div class="clear"></div>
        </div>
        <p id="last_build_software" class="last_build"></p><br/>
        <div class="clear"></div>
        <p>SlapOS rebuilds your software from source, allowing you to easily patch or add any free software. <a href="{{ url_for('viewLog', logfile='software.log') }}">Learn how!</a></p>
      </div>

      <h2 class="instance">Running State</h2>
      <div class="log_content_box">
        <!--<div id="instance_run_state" class="state_terminated">-->
        <!--<div id="instance_run_state" class="state_stopped">-->
        <div id="instance_run_state" class="state_waiting">
          <span class="legend"></span>
          <p>Waiting for starting</p>
          <div class="clear"></div>
        </div>
        <p id="last_build_instance" class="last_build"></p><br/>
        <div class="clear"></div>
        <p>SlapOS configures your running environment and processes to match your needs. <a href="{{ url_for('viewLog', logfile='instance.log') }}">Learn how!</a></p>
      </div>

      <div class="separator"></div>
      <div style="height: 38px; text-align: center; padding-top: 25px;">
        <a href="#" id="slapswitch" class="lshare" rel="stop">STOP PROCESS</a>
      </div>
    </div>
    <div id="openloglist">
      <h2  class="boxtitle">Select log file to view</h2>
      <div style="height: 340px; margin-bottom: 10px;">
        <div id='addbox' style="display:none">
          <a href="#inline_content" id="inlineViewer" class="add_btn">ADD FILE</a>
          <div class="separator"></div>
        </div>
        <ul>
          <li rel='software.log'>Software.log</li>
          <li rel='instance.log' class='checked'>Instance.log</li>
         </ul>
      </div>
      <div class="separator"></div>
      <div style="height: 38px; text-align: left; padding: 20px 0 0 15px;">
        <a id="logEdit" class="lshare no-right-border" style="float:left">Edit list</a>
        <a id="refreshlog" class="lshare" style="float:left">Refresh log</a>
      </div>
    </div>
  </div>
</div>
<div class='clear'></div>

<!-- This contains the hidden content for inline calls -->
<div style='display:none'>
  <div id='inline_content' style='padding:10px; background:#fff;'>
    <div style='border: solid 1px #678dad; height: 300px; overflow: auto;'>
      <div id="fileTree"></div>
    </div>
    <input type=submit value="Add file" id="addfile" class="button">
  </div>
</div>
{% endblock %}
